@@ -15,109 +15,70 @@
 # limitations under the License.
 #
 
-from warnings import warn
 from pyspark import keyword_only
+
 from ai.h2o.sparkling import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
-from ai.h2o.sparkling.ml.algos.H2OSupervisedAlgorithmWithFoldColumn import H2OSupervisedAlgorithmWithFoldColumn
-from ai.h2o.sparkling.ml.models.H2OGLMMOJOModel import H2OGLMMOJOModel
-from ai.h2o.sparkling.ml.params.H2OGLMParams import H2OGLMParams
-from ai.h2o.sparkling.ml.algos.FamilyBasedH2OTrainFramePreparation import FamilyBasedH2OTrainFramePreparation
+from ai.h2o.sparkling.ml.algos.H2OSupervisedAlgoBase import H2OSupervisedAlgoBase
+from ai.h2o.sparkling.ml.params import H2OGLMParams
 
 
-class H2OGLM(H2OGLMParams, H2OSupervisedAlgorithmWithFoldColumn, FamilyBasedH2OTrainFramePreparation):
+class H2OGLM(H2OGLMParams, H2OSupervisedAlgoBase):
 
     @keyword_only
     def __init__(self,
-                 randomCols=None,
-                 ignoredCols=None,
-                 plugValues=None,
-                 betaConstraints=None,
-                 interactionPairs=None,
-                 columnsToCategorical=[],
-                 keepBinaryModels=False,
-                 withContributions=False,
-                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
-                 withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
-                 convertInvalidNumbersToNa=False,
-                 detailedPredictionCol="detailed_prediction",
-                 validationDataFrame=None,
-                 featuresCols=[],
-                 predictionCol="prediction",
-                 convertUnknownCategoricalLevelsToNa=False,
-                 splitRatio=1.0,
-                 withStageResults=False,
-                 seed=-1,
-                 family="AUTO",
-                 randomFamily=None,
-                 tweedieVariancePower=0.0,
-                 tweedieLinkPower=1.0,
-                 theta=1.0E-10,
+                 standardize=True,
+                 family="gaussian",
+                 link="family_default",
                  solver="AUTO",
+                 tweedieVariancePower=0.0,
+                 tweedieLinkPower=0.0,
                  alphaValue=None,
                  lambdaValue=None,
-                 lambdaSearch=False,
-                 earlyStopping=True,
-                 nlambdas=-1,
-                 scoreIterationInterval=-1,
-                 standardize=True,
-                 coldStart=False,
                  missingValuesHandling="MeanImputation",
+                 prior=-1.0,
+                 lambdaSearch=False,
+                 nlambdas=-1,
                  nonNegative=False,
+                 lambdaMinRatio=-1.0,
                  maxIterations=-1,
-                 betaEpsilon=1.0E-4,
+                 intercept=True,
+                 betaEpsilon=1e-4,
                  objectiveEpsilon=-1.0,
                  gradientEpsilon=-1.0,
                  objReg=-1.0,
-                 link="family_default",
-                 randomLink=None,
-                 startval=None,
-                 calcLike=False,
-                 intercept=True,
-                 HGLM=False,
-                 prior=-1.0,
-                 lambdaMinRatio=-1.0,
-                 maxActivePredictors=-1,
-                 interactions=None,
-                 balanceClasses=False,
-                 classSamplingFactors=None,
-                 maxAfterBalanceSize=5.0,
-                 maxConfusionMatrixSize=20,
                  computePValues=False,
                  removeCollinearCols=False,
-                 generateScoringHistory=False,
+                 interactions=None,
+                 interactionPairs=None,
+                 earlyStopping=True,
                  modelId=None,
-                 nfolds=0,
-                 keepCrossValidationModels=True,
                  keepCrossValidationPredictions=False,
                  keepCrossValidationFoldAssignment=False,
+                 parallelizeCrossValidation=True,
+                 distribution="AUTO",
                  labelCol="label",
+                 foldCol=None,
                  weightCol=None,
                  offsetCol=None,
-                 foldCol=None,
-                 foldAssignment="AUTO",
-                 ignoreConstCols=True,
-                 scoreEachIteration=False,
-                 checkpoint=None,
-                 stoppingRounds=0,
-                 maxRuntimeSecs=0.0,
-                 stoppingMetric="AUTO",
-                 stoppingTolerance=0.001,
-                 customMetricFunc=None,
-                 exportCheckpointsDir=None,
-                 aucType="AUTO"):
+                 splitRatio=1.0,
+                 seed=-1,
+                 nfolds=0,
+                 allStringColumnsToCategorical=True,
+                 columnsToCategorical=[],
+                 predictionCol="prediction",
+                 detailedPredictionCol="detailed_prediction",
+                 withDetailedPredictionCol=True,
+                 featuresCols=[],
+                 convertUnknownCategoricalLevelsToNa=False,
+                 convertInvalidNumbersToNa=False,
+                 namedMojoOutputColumns=True,
+                 withContributions=False,
+                 withLeafNodeAssignments=False,
+                 **DeprecatedParams):
         Initializer.load_sparkling_jar()
         super(H2OGLM, self).__init__()
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.H2OGLM", self.uid)
         self._setDefaultValuesFromJava()
         kwargs = Utils.getInputKwargs(self)
-        kwargs = self._updateInitKwargs(kwargs)
-
-        if 'interactionPairs' in kwargs:
-            warn("Interaction pairs are not supported!")
         self._set(**kwargs)
-        self._transfer_params_to_java()
-
-    def _create_model(self, javaModel):
-        return H2OGLMMOJOModel(javaModel)
