@@ -8,25 +8,23 @@ from h2o.utils.compatibility import *  # NOQA
 
 from datetime import datetime
 import inspect
+import types
 import warnings
 
 import h2o
-from h2o.base import Keyed
 from h2o.exceptions import H2OValueError, H2OResponseError
 from h2o.frame import H2OFrame
 from h2o.job import H2OJob
-from h2o.utils.mixin import assign, load_ext, mixin
 from h2o.utils.shared_utils import quoted
 from h2o.utils.typechecks import assert_is_type, is_type, numeric, FunctionType
 from ..model.autoencoder import H2OAutoEncoderModel
 from ..model.binomial import H2OBinomialModel
-from ..model.binomial_uplift import H2OBinomialUpliftModel
 from ..model.clustering import H2OClusteringModel
 from ..model.dim_reduction import H2ODimReductionModel, H2OTargetEncoderMetrics
 from ..model.metrics_base import (H2OBinomialModelMetrics, H2OClusteringModelMetrics, H2ORegressionModelMetrics,
                                   H2OMultinomialModelMetrics, H2OAutoEncoderModelMetrics, H2ODimReductionModelMetrics,
                                   H2OWordEmbeddingModelMetrics, H2OOrdinalModelMetrics, H2OAnomalyDetectionModelMetrics,
-                                  H2OModelMetricsRegressionCoxPH, H2OBinomialUpliftModelMetrics)
+                                  H2OCoxPHModelMetrics)
 from ..model.model_base import ModelBase
 from ..model.multinomial import H2OMultinomialModel
 from ..model.ordinal import H2OOrdinalModel
@@ -34,7 +32,6 @@ from ..model.regression import H2ORegressionModel
 from ..model.word_embedding import H2OWordEmbeddingModel
 from ..model.anomaly_detection import H2OAnomalyDetectionModel
 from ..model.coxph import H2OCoxPHModel
-from ..model.coxph import H2OCoxPHMojoModel
 from ..model.segment_models import H2OSegmentModels
 
 
@@ -56,13 +53,6 @@ class H2OEstimator(ModelBase):
 
     Because H2OEstimator instances are instances of ModelBase, these objects can use the H2O model API.
     """
-    
-    supervised_learning = None  # overridden in implementation
-    __default_params = None     # mangled/hidden: use `_default_param_value` to access default param values internally.
-
-    def __init__(self):
-        super(H2OEstimator, self).__init__()
-        self._model = self
 
     def start(self, x, y=None, training_frame=None, offset_column=None, fold_column=None,
               weights_column=None, validation_frame=None, **params):
@@ -89,6 +79,7 @@ class H2OEstimator(ModelBase):
                    validation_frame=validation_frame,
                    **params)
 
+
     def join(self):
         """Wait until job's completion."""
         self._future = False
@@ -97,7 +88,8 @@ class H2OEstimator(ModelBase):
         self._job = None
         model_json = h2o.api("GET /%d/Models/%s" % (self._rest_version, model_key))["models"][0]
         self._resolve_model(model_key, model_json)
-    
+
+
     def train(self, x=None, y=None, training_frame=None, offset_column=None, fold_column=None,
               weights_column=None, validation_frame=None, max_runtime_secs=None, ignored_columns=None,
               model_id=None, verbose=False):
@@ -121,7 +113,6 @@ class H2OEstimator(ModelBase):
                                  validation_frame=validation_frame, max_runtime_secs=max_runtime_secs, 
                                  ignored_columns=ignored_columns, model_id=model_id, verbose=verbose)
         self._train(parms, verbose=verbose)
-        return self
 
     def train_segments(self, x=None, y=None, training_frame=None, offset_column=None, fold_column=None,
                        weights_column=None, validation_frame=None, max_runtime_secs=None, ignored_columns=None,
@@ -182,7 +173,7 @@ class H2OEstimator(ModelBase):
                                  ignored_columns=ignored_columns, model_id=None, verbose=verbose)
 
         if isinstance(segments, H2OFrame):
-            parms["segments"] = H2OEstimator._keyify(segments)
+            parms["segments"] = H2OEstimator._keyify_if_h2oframe(segments)
         else:
             parms["segment_columns"] = segments
         if segment_models_id:
@@ -195,6 +186,7 @@ class H2OEstimator(ModelBase):
         job.poll()
         return H2OSegmentModels(job.dest_key)
 
+
     def _train(self, parms, verbose=False):
         assert_is_type(verbose, bool)
 
@@ -216,15 +208,16 @@ class H2OEstimator(ModelBase):
         model_json = h2o.api("GET /%d/Models/%s" % (rest_ver, job.dest_key))["models"][0]
         self._resolve_model(job.dest_key, model_json)
 
+
     def _make_parms(self, x=None, y=None, training_frame=None, offset_column=None, fold_column=None,
                     weights_column=None, validation_frame=None, max_runtime_secs=None, ignored_columns=None,
                     model_id=None, verbose=False, extend_parms_fn=None):
         has_default_training_frame = hasattr(self, 'training_frame') and self.training_frame is not None
         training_frame = H2OFrame._validate(training_frame, 'training_frame',
-                                            required=self._options_.get('requires_training_frame', True) and not has_default_training_frame)
+                                            required=self._requires_training_frame() and not has_default_training_frame)
         validation_frame = H2OFrame._validate(validation_frame, 'validation_frame')
         assert_is_type(y, None, int, str)
-        assert_is_type(x, None, int, str, ModelBase, [str, int], {str, int})
+        assert_is_type(x, None, int, str, [str, int], {str, int})
         assert_is_type(ignored_columns, None, [str, int], {str, int})
         assert_is_type(offset_column, None, int, str)
         assert_is_type(fold_column, None, int, str)
@@ -239,14 +232,25 @@ class H2OEstimator(ModelBase):
             self._verify_training_frame_params(offset_column, fold_column, weights_column, validation_frame)
             training_frame = self.training_frame if has_default_training_frame else None
     
-        if verbose and not self._options_.get('verbose', False):
-            raise H2OValueError("Verbose mode is not available for %s" % self.__class__.__name__)
+        algo = self.algo
+        if verbose and algo not in ["drf", "gbm", "deeplearning", "xgboost"]:
+            raise H2OValueError("Verbose should only be set to True for drf, gbm, deeplearning, and xgboost models")
         parms = self._parms.copy()
+        if algo=="pca" and "k" not in parms.keys():
+            parms["k"] = 1
+        if "__class__" in parms:  # FIXME: hackt for PY3
+            del parms["__class__"]
+        is_auto_encoder = bool(parms.get("autoencoder"))
+        is_supervised = not(is_auto_encoder or algo in {"aggregator", "pca", "svd", "kmeans", "glrm", "word2vec", "isolationforest", "generic"})
+    
         names = training_frame.names if training_frame is not None else []
         ncols = training_frame.ncols if training_frame is not None else 0
         types = training_frame.types if training_frame is not None else {}
     
-        if self.supervised_learning:
+        if "checkpoint" in parms and isinstance(parms["checkpoint"], H2OEstimator):
+            parms["checkpoint"] = parms["checkpoint"].key
+    
+        if is_supervised:
             if y is None: y = "response"
             if is_type(y, int):
                 if not (-ncols <= y < ncols):
@@ -261,7 +265,6 @@ class H2OEstimator(ModelBase):
             # it. This way an unsupervised model can be used as a step in
             # sklearn's pipeline.
             y = None
-            self._estimator_type = "unsupervised"
     
         if override_default_training_frame:
             assert_is_type(y, str, None)
@@ -282,8 +285,6 @@ class H2OEstimator(ModelBase):
                         ignored_columns_set.add(ic)
             if x is None:
                 xset = set(names) - {y} - ignored_columns_set
-            elif isinstance(x, ModelBase) and hasattr(x, '_extract_x_from_model'):
-                xset = x._extract_x_from_model()
             else:
                 xset = set()
                 if is_type(x, int, str): x = [x]
@@ -306,6 +307,16 @@ class H2OEstimator(ModelBase):
         # Overwrites the model_id parameter only if model_id is passed
         if model_id is not None:
             parms["model_id"] = model_id
+    
+        # Step 2
+        is_auto_encoder = "autoencoder" in parms and parms["autoencoder"]
+        is_unsupervised = is_auto_encoder or self.algo in {"aggregator", "pca", "svd", "kmeans", "glrm", "word2vec", "isolationforest"}
+        if is_auto_encoder and y is not None:
+            raise ValueError("y should not be specified for autoencoder.")
+        if not is_unsupervised and y is None and self.algo not in ["generic"]:
+            raise ValueError("Missing response")
+    
+        # Step 3
         if override_default_training_frame:
             parms["training_frame"] = training_frame
             offset = parms["offset_column"]
@@ -321,10 +332,10 @@ class H2OEstimator(ModelBase):
             parms["response_column"] = y
         if not isinstance(x, (list, tuple)):
             x = [x]
-        if len(x) > 0 and is_type(x[0], int):
+        if is_type(x[0], int):
             x = [names[i] for i in x]
         if override_default_training_frame:
-            ignored_columns = list(set(names) - set(x + [y, offset, folds, weights]))
+            ignored_columns = list(set(names) - set(x + [y, offset, folds, weights] + self._additional_used_columns(parms)))
             parms["ignored_columns"] = None if ignored_columns == [] else [quoted(col) for col in ignored_columns]
         parms["interactions"] = (None if "interactions" not in parms or parms["interactions"] is None
                                  else [quoted(col) for col in parms["interactions"]])
@@ -335,14 +346,16 @@ class H2OEstimator(ModelBase):
         if extend_parms_fn is not None:
             extend_parms_fn(parms)
     
-        parms = {k: H2OEstimator._keyify(v) for k, v in parms.items()}
+        parms = {k: H2OEstimator._keyify_if_h2oframe(parms[k]) for k in parms}
         if "r2" in (parms.get('stopping_metric') or []):
             raise H2OValueError("r2 cannot be used as an early stopping_metric yet.  Check this JIRA https://0xdata.atlassian.net/browse/PUBDEV-5381 for progress.")
         return parms
 
+
     def _get_rest_version(self, parms):
         return parms.pop("_rest_version") if "_rest_version" in parms else 3
 
+
     def _print_model_scoring_history(self, job, bar_progress=0):
         """
         the callback function used to poll/print updates during model training.
@@ -360,30 +373,35 @@ class H2OEstimator(ModelBase):
         except AttributeError:  # To catch NoneType error if scoring history is not available
             print("Scoring History is not available yet...")
 
+
     @staticmethod
-    def _keyify(item):
-        if isinstance(item, Keyed):
-            return item.key
-        elif isinstance(item, list) and any(isinstance(i, Keyed) for i in item):
-            return [quoted(H2OEstimator._keyify(i)) for i in item]
+    def _keyify_if_h2oframe(item):
+        if isinstance(item, H2OFrame):
+            return item.frame_id
+        elif isinstance(item, list) and all(i is None or isinstance(i, H2OFrame) for i in item):
+            return [quoted(i) if i is None else quoted(i.frame_id) for i in item]
         else:
             return item
 
+
     def _resolve_model(self, model_id, model_json):
         metrics_class, model_class, metrics_class_valid = H2OEstimator._metrics_class(model_json)
-        m = model_class() if model_class else self._model
+        m = model_class()
         m._id = model_id
         m._model_json = model_json
-        m._have_pojo = model_json.get('have_pojo', True)
-        m._have_mojo = model_json.get('have_mojo', True)
-        m._start_time = model_json.get('output', {}).get('start_time', None)
-        m._end_time = model_json.get('output', {}).get('end_time', None)
-        m._run_time = model_json.get('output', {}).get('run_time', None)
+        if (model_json["algo"]=="glm") and self.HGLM:
+            m._have_pojo = False
+            m._have_mojo = False
+        else:
+            m._have_pojo = model_json.get('have_pojo', True)
+            m._have_mojo = model_json.get('have_mojo', True)
         m._metrics_class = metrics_class
         m._metrics_class_valid = metrics_class_valid
         m._parms = self._parms
         m._estimator_type = self._estimator_type
-        m._options_ = self._options_
+        m._start_time = model_json.get('output', {}).get('start_time', None)
+        m._end_time = model_json.get('output', {}).get('end_time', None)
+        m._run_time = model_json.get('output', {}).get('run_time', None)
 
         if model_id is not None and model_json is not None and metrics_class is not None:
             # build Metric objects out of each metrics
@@ -404,10 +422,36 @@ class H2OEstimator(ModelBase):
             # build a useful dict of the params
             for p in m._model_json["parameters"]:
                 m.parms[p["name"]] = p
-                
-        extensions = [load_ext(ext) for ext in self._options_.get('model_extensions', [])]
-        mixin(self._model, model_class, *extensions)
-        assign(self._model, m)
+        H2OEstimator.mixin(self, model_class)
+        self.__dict__.update(m.__dict__.copy())
+
+
+    # TODO: replace with a property which is overriden in subclasses
+    def _compute_algo(self):
+        name = self.__class__.__name__
+        if name == "H2ODeepLearningEstimator": return "deeplearning"
+        if name == "H2OAutoEncoderEstimator": return "deeplearning"
+        if name == "H2OGradientBoostingEstimator": return "gbm"
+        if name == "H2OGeneralizedLinearEstimator": return "glm"
+        if name == "H2OGeneralizedLowRankEstimator": return "glrm"
+        if name == "H2OKMeansEstimator": return "kmeans"
+        if name == "H2ONaiveBayesEstimator": return "naivebayes"
+        if name == "H2ORandomForestEstimator": return "drf"
+        if name == "H2OXGBoostEstimator": return "xgboost"
+        if name == "H2OCoxProportionalHazardsEstimator": return "coxph"
+        if name == "H2OGeneralizedAdditiveEstimator": return "gam"
+        if name == "H2OIsolationForestEstimator": return "isolationforest"
+        if name in ["H2OPCA", "H2OPrincipalComponentAnalysisEstimator"]: return "pca"
+        if name in ["H2OSVD", "H2OSingularValueDecompositionEstimator"]: return "svd"
+
+
+    @staticmethod
+    def mixin(obj, cls):
+        for name in cls.__dict__:
+            if name.startswith("__") and name.endswith("__"): continue
+            if not isinstance(cls.__dict__[name], types.FunctionType): continue
+            obj.__dict__[name] = cls.__dict__[name].__get__(obj)
+
 
     #------ Scikit-learn Interface Methods -------
 
@@ -438,6 +482,7 @@ class H2OEstimator(ModelBase):
         self.train(x, y, training_frame, **params)
         return self
 
+
     def get_params(self, deep=True):
         """
         Obtain parameters for this estimator.
@@ -457,6 +502,7 @@ class H2OEstimator(ModelBase):
             out[key] = value
         return out
 
+
     def set_params(self, **parms):
         """
         Used by sklearn for updating parameters during grid search.
@@ -472,6 +518,20 @@ class H2OEstimator(ModelBase):
             if param is not None:
                 raise H2OValueError("No training frame defined, yet the parameter %d is has been specified.", param)
 
+    def _requires_training_frame(self):
+        """
+        Determines if a training frame is required for given algorithm.
+        :return: True as a default value. Can be overridden by any specific algorithm.
+        """
+        return True
+
+    def _additional_used_columns(self, parms):
+        """
+        Returns list of additional columns not to automatically add to ignored_columns parameter.
+        :return: Empty list as default. Can be overridden by any specific algorithm.
+        """
+        return []
+
     @staticmethod
     def _metrics_class(model_json):
         model_type = model_json["output"]["model_category"]
@@ -479,9 +539,6 @@ class H2OEstimator(ModelBase):
         if model_type == "Binomial":
             metrics_class = H2OBinomialModelMetrics
             model_class = H2OBinomialModel
-        elif model_type == "BinomialUplift":
-            metrics_class = H2OBinomialUpliftModelMetrics
-            model_class = H2OBinomialUpliftModel
         elif model_type == "Clustering":
             metrics_class = H2OClusteringModelMetrics
             model_class = H2OClusteringModel
@@ -508,40 +565,71 @@ class H2OEstimator(ModelBase):
             valid_metrics_class = H2OBinomialModelMetrics
             model_class = H2OAnomalyDetectionModel
         elif model_type == "CoxPH":
-            metrics_class = H2OModelMetricsRegressionCoxPH
-            model_class = H2OCoxPHModel if model_json["algo"] != "generic" else H2OCoxPHMojoModel
+            metrics_class = H2OCoxPHModelMetrics
+            model_class = H2OCoxPHModel
         elif model_type == "TargetEncoder":
             metrics_class = H2OTargetEncoderMetrics
-            model_class = None
-        elif model_type == "Unknown":
-            metrics_class = None
-            model_class = None
+            model_class = h2o.estimators.H2OTargetEncoderEstimator
         else:
             raise NotImplementedError(model_type)
         if valid_metrics_class is None:
             valid_metrics_class = metrics_class
         return [metrics_class, model_class, valid_metrics_class]
 
+    def convert_H2OXGBoostParams_2_XGBoostParams(self):
+        """
+        In order to use convert_H2OXGBoostParams_2_XGBoostParams and convert_H2OFrame_2_DMatrix, you must import
+        the following toolboxes: xgboost, pandas, numpy and scipy.sparse.
+
+        Given an H2OXGBoost model, this method will generate the corresponding parameters that should be used by
+        native XGBoost in order to give exactly the same result, assuming that the same dataset
+        (derived from h2oFrame) is used to train the native XGBoost model.
+
+        Follow the steps below to compare H2OXGBoost and native XGBoost:
+
+         1. Train the H2OXGBoost model with H2OFrame trainFile and generate a prediction:
+
+          - h2oModelD = H2OXGBoostEstimator(\*\*h2oParamsD) # parameters specified as a dict()
+          - h2oModelD.train(x=myX, y=y, training_frame=trainFile) # train with H2OFrame trainFile
+          - h2oPredict = h2oPredictD = h2oModelD.predict(trainFile)
+
+         2. Derive the DMatrix from H2OFrame:
+         
+          - nativeDMatrix = trainFile.convert_H2OFrame_2_DMatrix(myX, y, h2oModelD)
+
+         3. Derive the parameters for native XGBoost:
+         
+          - nativeParams = h2oModelD.convert_H2OXGBoostParams_2_XGBoostParams()
+
+         4. Train your native XGBoost model and generate a prediction:
+         
+          - nativeModel = xgb.train(params=nativeParams[0], dtrain=nativeDMatrix, num_boost_round=nativeParams[1])
+          - nativePredict = nativeModel.predict(data=nativeDMatrix, ntree_limit=nativeParams[1]
+
+         5. Compare the predictions h2oPredict from H2OXGBoost, nativePredict from native XGBoost.
+
+        :return: nativeParams, num_boost_round
+        """
+        import xgboost as xgb
+
+        nativeParams = self._model_json["output"]["native_parameters"]
+        nativeXGBoostParams = dict()
+
+        for (a,keyname,keyvalue) in nativeParams.cell_values:
+            nativeXGBoostParams[keyname]=keyvalue
+        paramsSet = self.full_parameters
+
+        return nativeXGBoostParams, paramsSet['ntrees']['actual_value']
+
     def _check_and_save_parm(self, parms, parameter_name, parameter_value):
         """
         If a parameter is not stored in parms dict save it there (even though the value is None).
-        Else check if the parameter has been already set during initialization of estimator. 
-            If yes, check the new value is the same or not.
-            If the values are different, set the last passed value to params dict and throw UserWarning.
+        Else check if the parameter has been already set during initialization of estimator. If yes, check the new value is the same or not. If the values are different, set the last passed value to params dict and throw UserWarning.
         """
         if parameter_name not in parms:
             parms[parameter_name] = parameter_value
         elif parameter_value is not None and parms[parameter_name] != parameter_value:
-            prev_value = parms[parameter_name]
             parms[parameter_name] = parameter_value
-            if prev_value != self._default_param_value(parameter_name):
-                warnings.warn("\n\n\t`%s` parameter has been already set and had a different value in `train` method."
-                              " The last passed value \"%s\" is used." % (parameter_name, parameter_value), 
-                              UserWarning, 
-                              stacklevel=4)  # warning should refer to the original call to `train`, `train_xxx`
+            warnings.warn("\n\n\t`%s` parameter has been already set and had a different value in `train` method. The last passed value \"%s\" is used." % (parameter_name, parameter_value), UserWarning, stacklevel=2)
+
 
-    @classmethod
-    def _default_param_value(cls, param_name):
-        if cls.__default_params is None:
-            cls.__default_params = cls()
-        return getattr(cls.__default_params, param_name)
