@@ -1,6 +1,8 @@
 import math
 
 import h2o
+from h2o.estimators import H2OXGBoostEstimator
+from h2o.utils.metaclass import Deprecated as deprecated
 
 
 class H2OTree(object):
@@ -49,7 +51,6 @@ class H2OTree(object):
     :param model: The model this tree is related to.
     :param tree_number: An integer representing the order in which the tree has been built in the model.
     :param tree_class: A string representing the name of the tree's class. Specifies the class of the tree requested. Required for multi-class classification. The number of tree classes equals the number of levels in categorical response column. As there is exactly one class per categorical level, the name of the tree's class is equal to the corresponding categorical level of the response column. In case of regression and binomial models, the name of the categorical level is ignored and can be omitted.
-    :param plain_language_rules: (Optional) Whether to generate plain language rules. "AUTO" by default, meaning False for big trees and True for small trees.
 
     :examples:
     
@@ -68,11 +69,10 @@ class H2OTree(object):
     
     """
 
-    def __init__(self, model, tree_number, tree_class=None, plain_language_rules="AUTO"):
+    def __init__(self, model, tree_number, tree_class=None):
         params = {"model": model.model_id,
                   "tree_number": tree_number,
-                  "tree_class": tree_class,
-                  "plain_language_rules": plain_language_rules}
+                  "tree_class": tree_class}
         response = h2o.api(endpoint="GET /3/Tree", data=params)
 
         self._left_children = response['left_children']
@@ -88,16 +88,8 @@ class H2OTree(object):
         self._nas = response['nas']
         self._predictions = response['predictions']
         self._root_node = self.__assemble_tree(0)
-        if response['tree_decision_path'] is None:
-            self._tree_decision_path = "Plain language rules generation is turned off."
-            self._decision_paths = "Plain language rules generation is turned off."
-        else:
-            self._tree_decision_path = response['tree_decision_path']
-            self._decision_paths = response['decision_paths']
-            
-        (left, right) = self.__per_node_cat_splits()
-        self._left_cat_split = left
-        self._right_cat_split = right
+        self._tree_decision_path = response['tree_decision_path']
+        self._decision_paths = response['decision_paths']
 
     @property
     def left_children(self):
@@ -411,7 +403,7 @@ class H2OTree(object):
     def __decode_categoricals(self, model, levels):
         string_levels = len(self._left_children) * [None]
 
-        if model.algo == 'xgboost':
+        if type(model) is H2OXGBoostEstimator:
             return string_levels
 
         for i in range(0, len(self._left_children)):
@@ -460,41 +452,6 @@ class H2OTree(object):
 
         return node_ids
 
-    def __per_node_cat_splits(self):
-
-        num_records = len(self.left_children)
-        per_node_levels_left = [None] * num_records
-        per_node_levels_right = [None] * num_records
-        for i in range(0, (len(self.left_children))):
-
-            # -1 means leaf - if a node is leaf node, there is not child and thus no split
-            left_idx = self._left_children[i]
-            right_idx = self._right_children[i]
-
-            if (left_idx != -1):
-                per_node_levels_left[i] = (self.levels[left_idx])
-
-            if (right_idx != -1):
-                per_node_levels_right[i] = (self.levels[right_idx])
-
-        return (per_node_levels_left, per_node_levels_right)
-
-    @property
-    def left_cat_split(self):
-        """
-        :return: Categorical levels leading to the left child node. Only present when split is categorical, otherwise none.
-        """
-        return self._left_cat_split
-
-    @property
-    def right_cat_split(self):
-        """
-        
-        :return: Categorical levels leading to the right child node. Only present when split is categorical, otherwise none.
- 
-        """
-        return self._right_cat_split
-
     def __len__(self):
         """
         Returns number of nodes inside the tree.
